clear all;close all;clc %#ok<*CLALL>
% Matlab script built by Joost de Winter, Lars Kooijman and Pavlo Bazilinksyy <p.bazilinskyy@tudelft.nl>
% Read data from three clients saved on the host
HostN=readmatrix('World Root-HostLog2019_09_26_19_44_46.csv');
%% Interpolate and filter data
M=[55    56    57    61    62    63    67    68    69    73    74    75    79    80    81    85    86    87    91    92    93    97    98    99   103   104   105   109   110   111   115   116   117   133   134   135 ...
   139   140   141   145   146   147   151   152   153   157   158   159   163   164   165   169   170   171   175   176   177   181   182   183   187   188   189]; % vector of relevant motion suit variables (angles only, exclude angles which give zeros all the time)
HostN(:,M)=unwrap(deg2rad(HostN(:,M))); % remove phase shifts using the unwrap function. First convert the angles from degrees to radians
fs = 100; % desired sampling frequency for interpolation
HostN=interp1(HostN(:,1),HostN,1/fs:1/fs:max(HostN(:,1))); % Interpolate all data to fs Hz to that the filter works properly
[b,a]               = butter(3,(1/(fs/2)),'Low');           % Low-pass filter (1 Hz)
HostN(:,[2:7 9:26 28:39]) = filtfilt(b,a,HostN(:,[2:7 9:26 28:39]));   % Filter the data of the agents
[b,a]               = butter(3,(10/(fs/2)),'Low');         % Low-pass filter (10 Hz)
HostN(:,40:end)     = filtfilt(b,a,HostN(:,40:end));       % Filter the data of the motion suit at a higher frequency (10 Hz)
%% Compute vehicle speed and agent activity
Driver1S=3.6*100*sqrt(diff(HostN(:,2)).^2+diff(HostN(:,4)).^2); % compute speed of manually driven car in kph (it is calculated by differtiating the position, presumably a better approach that Piotr's smoothed values, because filtfilt uses a zero-phase filter)
Driver2S=3.6*100*sqrt(diff(HostN(:,21)).^2+diff(HostN(:,23)).^2); % compute speed of automated car in kph
PedestrianRot=abs((1/fs)*diff(HostN(:,M))); % calculate absolute angular velocities of the relevant motion suit angular variables (this should be in radians per second)
[~,~,~,~,pedestrianactivity]=factoran(PedestrianRot,1); % maximum likelihood factor analysis, extracting 1 factor. pedestrianactivity is represented by the factor scores
pedestrianactivity=zscore(pedestrianactivity); % compute zscores (i.e., mean = 0, SD = 1)
%% Create figure showing the paths of agents on top of view screenshot
figure;hold on;box on;grid on
S=31400;E=40000; % start and end index of data points for plotting (time from 314.00 s to 400.00 s, 86 seconds in total)
plot(HostN(S:E,2),HostN(S:E,4),'c-','Linewidth',7); % plot path of manually driven car
plot(HostN(S:E,21),HostN(S:E,23),'m-','Linewidth',7); % plot path of automated car
plot(HostN(S:E,46),HostN(S:E,48),'-','color',[255 165 0]/255,'Linewidth',7); % plot path of pedestrian
I=imread('topview.png'); % read the top view (orthographic projection)
I=imrotate(I,180);I=fliplr(I); % rotate and flip amage for plotting
h=image([-262.620586989859 48.6906808643446],[-190.068343215629 165.422796927353],I); % dimensions of the image are obtained through manual inspection and trial and error
uistack(h,'bottom') % put image at the bottom, plotted lines should be on top
xlabel('x (m)'),ylabel('z (m)')
legend('Manually driven car','Automated car','Pedestrian','location','northwest')
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',26)
set(gca,'TickDir','out','LooseInset',[0.01 0.01 0.01 0.01],'xlim',[-125 50],'ylim',[-50 150])
axis equal % preserve the horizontal/vertical scaling of the map
%% Create figure showing the speed/activity of the agents
figure;hold on;box on;grid on
plot(HostN(S:E,1)-HostN(S),Driver1S(S:E),'c-','Linewidth',3); % plot speed of manually driven car (left axis)
plot(HostN(S:E,1)-HostN(S),Driver2S(S:E),'m-','Linewidth',3); % plot speed of automated car (left axis)
ylabel('Vehicle speed (km/h)')
yyaxis right;plot(HostN(S:E,1)-HostN(S),pedestrianactivity(S:E,:),'color',[255 165 0]/255,'Linewidth',3); % plot pedestrian activity (right axis)
xlabel('Time (s)');ylabel('Pedestrian activity (z-score)')
legend('Manually driven car','Automated car','Pedestrian','location','northeast')
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',26)
set(gca,'TickDir','out','LooseInset',[0.01 0.01 0.01 0.01],'xlim',[0 40])
%% Create figure showing the TTC of the cars, and pedestrian elbow angle
distance_manualcar_pedestrian=sqrt((HostN(:,2)-HostN(:,46)).^2+(HostN(:,4)-HostN(:,48)).^2);
distance_automatedcar_pedestrian=sqrt((HostN(:,21)-HostN(:,46)).^2+(HostN(:,23)-HostN(:,48)).^2);
figure;hold on;box on;grid on
plot(HostN(S:E,1)-HostN(S),distance_manualcar_pedestrian(S:E)./(Driver1S(S:E)/3.6),'c-','Linewidth',3); % plot TTC for manual car
plot(HostN(S:E,1)-HostN(S),distance_automatedcar_pedestrian(S:E)./(Driver2S(S:E)/3.6),'-','Linewidth',3); % plot TTC for automated car
set(gca,'ylim',[0 20])
xlabel('Time (s)');ylabel('Vehicle''s time to collision with pedestrian (s)')
yyaxis right;
plot(HostN(S:E,1)-HostN(S),-(rad2deg(HostN(S:E,146))-360),'color',[255 165 0]/255,'Linewidth',3);hold on % plot pedestrian activity (right axis)
plot(HostN(S:E,1)-HostN(S),-(rad2deg(HostN(S:E,182))-360),':','color',[255 165 0]/255,'Linewidth',3); % plot pedestrian activity (right axis)
xlabel('Time (s)');ylabel('Elbow up-down angle (deg)')
legend('Manually driven car','Automated car','Pedestrian left elbow','Pedestrian right elbow','location','northwest')
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',26)
set(gca,'TickDir','out','LooseInset',[0.01 0.01 0.01 0.01],'xlim',[5 26])
%%
% Definitions of variables in HostN
% 1	Timestamp
% 2	Driver0		pos_x
% 3	Driver0		pos_y
% 4	Driver0		pos_z
% 5	Driver0		rot_x
% 6	Driver0		rot_y
% 7	Driver0		rot_z
% 8	Driver0		blinkers
% 9	Driver0		vel_local_x
% 10	Driver0		vel_local_y
% 11	Driver0		vel_local_z
% 12	Driver0		vel_local_smooth_x
% 13	Driver0		vel_local_smooth_y
% 14	Driver0		vel_local_smooth_z
% 15	Driver0		vel_x
% 16	Driver0		vel_y
% 17	Driver0		vel_z
% 18	Driver0		vel_smooth_x
% 19	Driver0		vel_smooth_y
% 20	Driver0		vel_smooth_z
% 21	Driver1		pos_x
% 22	Driver1		pos_y
% 23	Driver1		pos_z
% 24	Driver1		rot_x
% 25	Driver1		rot_y
% 26	Driver1		rot_z
% 27	Driver1		blinkers
% 28	Driver1		vel_local_x
% 29	Driver1		vel_local_y
% 30	Driver1		vel_local_z
% 31	Driver1		vel_local_smooth_x
% 32	Driver1		vel_local_smooth_y
% 33	Driver1		vel_local_smooth_z
% 34	Driver1		vel_x
% 35	Driver1		vel_y
% 36	Driver1		vel_z
% 37	Driver1		vel_smooth_x
% 38	Driver1		vel_smooth_y
% 39	Driver1		vel_smooth_z
% 40	Pedestrian0	Root	pos_x
% 41	Pedestrian0	Root	pos_y
% 42	Pedestrian0	Root	pos_z
% 43	Pedestrian0	Root	rot_x
% 44	Pedestrian0	Root	rot_y
% 45	Pedestrian0	Root	rot_z
% 46	Pedestrian0	avatar	pos_x
% 47	Pedestrian0	avatar	pos_y
% 48	Pedestrian0	avatar	pos_z
% 49	Pedestrian0	avatar	rot_x
% 50	Pedestrian0	avatar	rot_y
% 51	Pedestrian0	avatar	rot_z
% 52	Pedestrian0	Hips	pos_x
% 53	Pedestrian0	Hips	pos_y
% 54	Pedestrian0	Hips	pos_z
% 55	Pedestrian0	Hips	rot_x
% 56	Pedestrian0	Hips	rot_y
% 57	Pedestrian0	Hips	rot_z
% 58	Pedestrian0	LeftHip	pos_x
% 59	Pedestrian0	LeftHip	pos_y
% 60	Pedestrian0	LeftHip	pos_z
% 61	Pedestrian0	LeftHip	rot_x
% 62	Pedestrian0	LeftHip	rot_y
% 63	Pedestrian0	LeftHip	rot_z
% 64	Pedestrian0	LeftKnee	pos_x
% 65	Pedestrian0	LeftKnee	pos_y
% 66	Pedestrian0	LeftKnee	pos_z
% 67	Pedestrian0	LeftKnee	rot_x
% 68	Pedestrian0	LeftKnee	rot_y
% 69	Pedestrian0	LeftKnee	rot_z
% 70	Pedestrian0	LeftAnkle	pos_x
% 71	Pedestrian0	LeftAnkle	pos_y
% 72	Pedestrian0	LeftAnkle	pos_z
% 73	Pedestrian0	LeftAnkle	rot_x
% 74	Pedestrian0	LeftAnkle	rot_y
% 75	Pedestrian0	LeftAnkle	rot_z
% 76	Pedestrian0	LeftToe 1	pos_x
% 77	Pedestrian0	LeftToe 1	pos_y
% 78	Pedestrian0	LeftToe 1	pos_z
% 79	Pedestrian0	LeftToe 1	rot_x
% 80	Pedestrian0	LeftToe 1	rot_y
% 81	Pedestrian0	LeftToe 1	rot_z
% 82	Pedestrian0	RightHip	pos_x
% 83	Pedestrian0	RightHip	pos_y
% 84	Pedestrian0	RightHip	pos_z
% 85	Pedestrian0	RightHip	rot_x
% 86	Pedestrian0	RightHip	rot_y
% 87	Pedestrian0	RightHip	rot_z
% 88	Pedestrian0	RightKnee	pos_x
% 89	Pedestrian0	RightKnee	pos_y
% 90	Pedestrian0	RightKnee	pos_z
% 91	Pedestrian0	RightKnee	rot_x
% 92	Pedestrian0	RightKnee	rot_y
% 93	Pedestrian0	RightKnee	rot_z
% 94	Pedestrian0	RightAnkle	pos_x
% 95	Pedestrian0	RightAnkle	pos_y
% 96	Pedestrian0	RightAnkle	pos_z
% 97	Pedestrian0	RightAnkle	rot_x
% 98	Pedestrian0	RightAnkle	rot_y
% 99	Pedestrian0	RightAnkle	rot_z
% 100	Pedestrian0	RightToe 1	pos_x
% 101	Pedestrian0	RightToe 1	pos_y
% 102	Pedestrian0	RightToe 1	pos_z
% 103	Pedestrian0	RightToe 1	rot_x
% 104	Pedestrian0	RightToe 1	rot_y
% 105	Pedestrian0	RightToe 1	rot_z
% 106	Pedestrian0	Chest	pos_x
% 107	Pedestrian0	Chest	pos_y
% 108	Pedestrian0	Chest	pos_z
% 109	Pedestrian0	Chest	rot_x
% 110	Pedestrian0	Chest	rot_y
% 111	Pedestrian0	Chest	rot_z
% 112	Pedestrian0	Chest2	pos_x
% 113	Pedestrian0	Chest2	pos_y
% 114	Pedestrian0	Chest2	pos_z
% 115	Pedestrian0	Chest2	rot_x
% 116	Pedestrian0	Chest2	rot_y
% 117	Pedestrian0	Chest2	rot_z
% 118	Pedestrian0	Chest3	pos_x
% 119	Pedestrian0	Chest3	pos_y
% 120	Pedestrian0	Chest3	pos_z
% 121	Pedestrian0	Chest3	rot_x
% 122	Pedestrian0	Chest3	rot_y
% 123	Pedestrian0	Chest3	rot_z
% 124	Pedestrian0	Chest4	pos_x
% 125	Pedestrian0	Chest4	pos_y
% 126	Pedestrian0	Chest4	pos_z
% 127	Pedestrian0	Chest4	rot_x
% 128	Pedestrian0	Chest4	rot_y
% 129	Pedestrian0	Chest4	rot_z
% 130	Pedestrian0	LeftCollar	pos_x
% 131	Pedestrian0	LeftCollar	pos_y
% 132	Pedestrian0	LeftCollar	pos_z
% 133	Pedestrian0	LeftCollar	rot_x
% 134	Pedestrian0	LeftCollar	rot_y
% 135	Pedestrian0	LeftCollar	rot_z
% 136	Pedestrian0	LeftShoulder 1	pos_x
% 137	Pedestrian0	LeftShoulder 1	pos_y
% 138	Pedestrian0	LeftShoulder 1	pos_z
% 139	Pedestrian0	LeftShoulder 1	rot_x
% 140	Pedestrian0	LeftShoulder 1	rot_y
% 141	Pedestrian0	LeftShoulder 1	rot_z
% 142	Pedestrian0	LeftElbow	pos_x
% 143	Pedestrian0	LeftElbow	pos_y
% 144	Pedestrian0	LeftElbow	pos_z
% 145	Pedestrian0	LeftElbow	rot_x
% 146	Pedestrian0	LeftElbow	rot_y
% 147	Pedestrian0	LeftElbow	rot_z
% 148	Pedestrian0	LeftCarpus 1	pos_x
% 149	Pedestrian0	LeftCarpus 1	pos_y
% 150	Pedestrian0	LeftCarpus 1	pos_z
% 151	Pedestrian0	LeftCarpus 1	rot_x
% 152	Pedestrian0	LeftCarpus 1	rot_y
% 153	Pedestrian0	LeftCarpus 1	rot_z
% 154	Pedestrian0	Neck 1	pos_x
% 155	Pedestrian0	Neck 1	pos_y
% 156	Pedestrian0	Neck 1	pos_z
% 157	Pedestrian0	Neck 1	rot_x
% 158	Pedestrian0	Neck 1	rot_y
% 159	Pedestrian0	Neck 1	rot_z
% 160	Pedestrian0	Head 1	pos_x
% 161	Pedestrian0	Head 1	pos_y
% 162	Pedestrian0	Head 1	pos_z
% 163	Pedestrian0	Head 1	rot_x
% 164	Pedestrian0	Head 1	rot_y
% 165	Pedestrian0	Head 1	rot_z
% 166	Pedestrian0	RightCollar	pos_x
% 167	Pedestrian0	RightCollar	pos_y
% 168	Pedestrian0	RightCollar	pos_z
% 169	Pedestrian0	RightCollar	rot_x
% 170	Pedestrian0	RightCollar	rot_y
% 171	Pedestrian0	RightCollar	rot_z
% 172	Pedestrian0	RightShoulder 1	pos_x
% 173	Pedestrian0	RightShoulder 1	pos_y
% 174	Pedestrian0	RightShoulder 1	pos_z
% 175	Pedestrian0	RightShoulder 1	rot_x
% 176	Pedestrian0	RightShoulder 1	rot_y
% 177	Pedestrian0	RightShoulder 1	rot_z
% 178	Pedestrian0	RightElbow	pos_x
% 179	Pedestrian0	RightElbow	pos_y
% 180	Pedestrian0	RightElbow	pos_z
% 181	Pedestrian0	RightElbow	rot_x
% 182	Pedestrian0	RightElbow	rot_y
% 183	Pedestrian0	RightElbow	rot_z
% 184	Pedestrian0	RightCarpus 1	pos_x
% 185	Pedestrian0	RightCarpus 1	pos_y
% 186	Pedestrian0	RightCarpus 1	pos_z
% 187	Pedestrian0	RightCarpus 1	rot_x
% 188	Pedestrian0	RightCarpus 1	rot_y
% 189	Pedestrian0	RightCarpus 1	rot_z
